# -*- coding: utf-8 -*-
from django.core.exceptions import ObjectDoesNotExist
from django.utils.text import slugify

from rest_framework import serializers

from . import models


class RatingRelatedField(serializers.RelatedField):
    """
    A custom field to use for the `rating` relationship.
    """

    def to_representation(self, value):
        try:
            return str(models.Rating.objects.get(id=value.id))
        except ObjectDoesNotExist:
            raise Exception('Rating Id: {0} not found!'.format(value.id))

    def to_internal_value(self, value):
        try:
            return models.Rating.objects.get(slug=slugify(value))
        except ObjectDoesNotExist:
            raise Exception('Rating: {0} not found!'.format(value))


class TagRelatedField(serializers.RelatedField):
    """
    A custom field to use for the `tag` relationship.
    """

    def to_representation(self, value):
        try:
            return str(models.Tag.objects.get(id=value.id))
        except ObjectDoesNotExist:
            raise Exception('Tag Id: {0} not found!'.format(value.id))

    def to_internal_value(self, value):
        try:
            return models.Tag.objects.get(slug=slugify(value))
        except ObjectDoesNotExist:
            raise Exception('Tag: {0} not found!'.format(value))



# -*- coding: utf-8 -*-
from django_filters.rest_framework import DjangoFilterBackend
from rest_framework import permissions, viewsets

from . import filters, models, serializers


class ActivityViewSet(viewsets.ModelViewSet):
    """ViewSet for viewing and editing User Activities."""

    queryset = models.Activity.objects.fast()
    permission_classes = (permissions.IsAuthenticated, )
    filter_backends = (DjangoFilterBackend, )
    filter_class = filters.ActivityFilter

    def get_serializer_class(self):
        if self.request.method in ['POST', 'PUT', 'PATCH']:
            return serializers.ActivitySerializerCreateOrEdit
        return serializers.ActivitySerializer


class CommentViewSet(viewsets.ModelViewSet):
    """ViewSet for viewing and editing User Comments."""

    queryset = models.Comment.objects.fast()
    permission_classes = (permissions.IsAuthenticated, )
    filter_backends = (DjangoFilterBackend, )
    filter_class = filters.CommentFilter

    def get_serializer_class(self):
        if self.request.method in ['POST', 'PUT', 'PATCH']:
            return serializers.CommentSerializerCreateOrEdit
        return serializers.CommentSerializer


class ReviewViewSet(viewsets.ModelViewSet):
    """ViewSet for viewing and editing User Reviews."""

    queryset = models.Review.objects.fast()
    permission_classes = (permissions.IsAuthenticated, )
    filter_backends = (DjangoFilterBackend, )
    filter_class = filters.ReviewFilter

    def get_serializer_class(self):
        if self.request.method in ['POST', 'PUT', 'PATCH']:
            return serializers.ReviewSerializerCreateOrEdit
        return serializers.ReviewSerializer



# -*- coding: utf-8 -*-
from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes.fields import GenericForeignKey, GenericRelation
from django.db import models
from django.utils.translation import ugettext_lazy as _

from genomix.models import TimeStampedLabelModel
from model_utils.models import TimeStampedModel

from . import choices, managers


class UserActivity(TimeStampedModel):
    """
    An abstract base class model that provides timestamps and auth information
    about who created, modified a user activity.
    """
    user = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        on_delete=models.CASCADE,
    )
    text = models.CharField(max_length=255)
    active = models.BooleanField(default=True)
    activities = GenericRelation('user_activities.Activity')
    tags = models.ManyToManyField('user_activities.Tag', blank=True)

    # Mandatory fields for generic relation
    # See: https://docs.djangoproject.com/en/1.11/ref/contrib/contenttypes/#generic-relations
    content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE)
    object_id = models.PositiveIntegerField()
    content_object = GenericForeignKey('content_type', 'object_id')

    class Meta:
        abstract = True


class Activity(TimeStampedModel):
    """Track user likes, votes, etc."""

    user = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        on_delete=models.CASCADE,
    )
    activity_type = models.SmallIntegerField(choices=choices.ACTIVITY_TYPES)
    active = models.BooleanField(default=True)

    # Mandatory fields for generic relation
    # See: https://docs.djangoproject.com/en/1.11/ref/contrib/contenttypes/#generic-relations
    content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE)
    object_id = models.PositiveIntegerField()
    content_object = GenericForeignKey('content_type', 'object_id')

    objects = managers.ActivityQuerySet.as_manager()

    class Meta:
        verbose_name = _('Activity')
        verbose_name_plural = _('Activities')
        unique_together = ('user', 'activity_type', 'content_type', 'object_id',)

    def __str__(self):
        return self.get_activity_type_display()


class Rating(TimeStampedLabelModel):
    """Ratings"""

    class Meta:
        verbose_name = _('Rating')
        verbose_name_plural = _('Ratings')

    def __str__(self):
        return self.label


class Tag(TimeStampedLabelModel):
    """Tags"""

    class Meta:
        verbose_name = _('Tag')
        verbose_name_plural = _('Tag')

    def __str__(self):
        return self.label


class Comment(UserActivity):
    """User comments."""

    comments = GenericRelation('user_activities.Comment')

    objects = managers.CommentQuerySet.as_manager()

    class Meta:
        verbose_name = _('Comment')
        verbose_name_plural = _('Comments')

    def __str__(self):
        return self.text


class Review(UserActivity):
    """User reviews."""

    rating = models.ForeignKey(
        'user_activities.Rating',
        related_name='reviews',
        on_delete=models.PROTECT,
        blank=True,
        null=True,
    )
    comments = GenericRelation('user_activities.Comment')

    objects = managers.ReviewQuerySet.as_manager()

    class Meta:
        verbose_name = _('Review')
        verbose_name_plural = _('Reviews')

    def __str__(self):
        return self.text



# -*- coding: utf-8 -*-
from rest_framework import routers

from . import viewsets


router = routers.SimpleRouter()
router.register(r'', viewsets.ActivityViewSet)
router.register(r'comments', viewsets.CommentViewSet)
router.register(r'reviews', viewsets.ReviewViewSet)

default_router = routers.DefaultRouter()
default_router.register(r'user-activities', viewsets.ActivityViewSet)
default_router.register(r'comments', viewsets.CommentViewSet)
default_router.register(r'reviews', viewsets.ReviewViewSet)

urlpatterns = default_router.urls



# -*- coding: utf-8 -*-
from django.db.models import CharField

import django_filters
from genomix.filters import DisplayChoiceFilter

from . import choices, models


class ActivityFilter(django_filters.rest_framework.FilterSet):

    activity_type = DisplayChoiceFilter(choices=choices.ACTIVITY_TYPES)

    class Meta:
        model = models.Activity
        fields = [
            'user',
            'user__username',
            'activity_type',
            'active',
            'content_type',
            'content_type__model',
            'object_id',
        ]
        filter_overrides = {
            CharField: {
                'filter_class': django_filters.CharFilter,
                'extra': lambda f: {
                    'lookup_expr': 'iexact',
                },
            }
        }


class CommentFilter(django_filters.rest_framework.FilterSet):

    tags = django_filters.ModelMultipleChoiceFilter(
        queryset=models.Tag.objects.all(),
        widget=django_filters.widgets.CSVWidget,
        help_text='Multiple values may be separated by commas.',
    )

    class Meta:
        model = models.Comment
        fields = [
            'user',
            'user__username',
            'active',
            'tags',
            'tags__label',
            'content_type',
            'content_type__model',
            'object_id',
        ]
        filter_overrides = {
            CharField: {
                'filter_class': django_filters.CharFilter,
                'extra': lambda f: {
                    'lookup_expr': 'iexact',
                },
            }
        }


class ReviewFilter(django_filters.rest_framework.FilterSet):

    tags = django_filters.ModelMultipleChoiceFilter(
        queryset=models.Tag.objects.all(),
        widget=django_filters.widgets.CSVWidget,
        help_text='Multiple values may be separated by commas.',
    )

    class Meta:
        model = models.Review
        fields = [
            'user',
            'user__username',
            'active',
            'tags',
            'tags__label',
            'content_type',
            'content_type__model',
            'object_id',
            'rating',
            'rating__label',
        ]
        filter_overrides = {
            CharField: {
                'filter_class': django_filters.CharFilter,
                'extra': lambda f: {
                    'lookup_expr': 'iexact',
                },
            }
        }



# -*- coding: utf-8 -*-
from django.db import models
from django.db.models import Count, Q


class ActivityQuerySet(models.QuerySet):

    def fast(self):
        return self.select_related('user').all()

    def aggregate_counts(self, content_type, object_id, active=True):
        return self.fast().filter(
            Q(content_type=content_type) &
            Q(object_id=object_id) &
            Q(active=True)
        ).values('activity_type').annotate(total=Count('activity_type'))

    def user_actions(self, user, content_type, object_id):
        return self.fast().filter(
            Q(user__id=user.id) &
            Q(content_type=content_type) &
            Q(object_id=object_id)
        ).values('id', 'active', 'activity_type').distinct()


class CommentQuerySet(models.QuerySet):

    def fast(self):
        return self.select_related('user').prefetch_related('tags').all()


class ReviewQuerySet(models.QuerySet):

    def fast(self):
        return self.select_related('user').select_related('rating').prefetch_related('tags').all()
